#!/usr/bin/perl

###
# Parse a Berkley DB file of Snort alerts, produced by
# snorta-txt2bdb and create a file with counts of occurrence of each network.

###
# Load necessary libraries.
require "init.pl";


###
# Initialize input alert database.
if (! -r $ALERTDB) {
    die "Cannot read $ALERTDB. Exiting.\n$!\n";
}
if (! tie(%ALERTDB, "DB_File", $ALERTDB)) {
    die "Could not initialize input alert database $ALERTDB:\n$!\n";
}

###
# Process every entry in the input alert database
$counter = 0;
$| = 1;
%ALERTNETS = ();
while (($id, $alert) = each(%ALERTDB)) {

    # Print progress indicator as records are processed
    if ($VERBOSE && (! ($counter % 500))) {
	print ".";
    }
    $counter++;

    # Parse database entries into distinct fields.
    ($timestamp, $name, $srchost, $srcport, $dsthost, $dstport) =
	split(/\*/, $alert);

    # Ignore port scans that sneaked into the alert file.
    next if ($name =~ /^spp_portscan:/);

    # Obtain destination  network (Class C) of the alert. If we match purely
    # by host we would miss slight variations in addresses such as
    # those in dial-up accounts.
    next if ($dsthost eq "");

    # Keep track of alerts per alert day.
    $ALERTNETS{$dsthost} .= "$id,";

}
print "\n" if ($VERBOSE);

# Group alerts by destination host and write them to separate files
# for each host to allow them to be later imported as a spreadsheet.
# Create a file with counts of occurrence of each destination host.
&byhost();

###
# Clean up and exit.
untie(%ALERTDB);




##
# Subroutine: Process records indexed by destination host.
sub byhost {
  my($name, $ids, @ids, $timestamp, $srchost, $srcport, $dsthost, $dstport);
  my($count);
  if (! open (SUMFILE, "> $LIBDIR/bydsthost.csv")) {
      die "Could not create $LIBDIR/bydsthost.csv:\n$!\n";
  }
  while (($host, $ids) = each(%ALERTNETS)) {

    # Obtain alert IDs associated with the destination host
    @ids = split(/,/, $ids);

    $count = $#ids + 1;
    if ($VERBOSE) {
	print $host , " ($count)...\n";
    }
    print SUMFILE "$host,$count\n";
  }
  close(SUMFILE);
}


