#!/usr/bin/perl

###
# Parse a Berkley DB file of Snort alerts, produced by
# snorta-txt2bdb and create directories of alerts
# indexed by the alert name.

###
# Load necessary libraries.
require "init.pl";


###
# Initialize input alert database.
if (! -r $ALERTDB) {
    die "Cannot read $ALERTDB. Exiting.\n$!\n";
}
if (! tie(%ALERTDB, "DB_File", $ALERTDB)) {
    die "Could not initialize input alert database $ALERTDB:\n$!\n";
}

###
# Process every entry in the input alert database,
$counter = 0;
$| = 1;
%ALERTNAMES = ();
while (($id, $alert) = each(%ALERTDB)) {

    # Print progress indicator as records are processed
    if ($VERBOSE && (! ($counter % 500))) {
	print ".";
    }
    $counter++;

    # Parse database entries into distinct fields.
    ($timestamp, $name, $srchost, $srcport, $dsthost, $dstport) =
	split(/\*/, $alert);

    # Ignore port scans that sneaked into the alert file.
    next if ($name =~ /^spp_portscan:/);

    # Keep track of alerts per unique alert name.
    $ALERTNAMES{$name} .= "$id,";

}
print "\n" if ($VERBOSE);

# Group alerts by alert names and write them to separate files
# for each alert name to allow them to be later imported as a spreadsheet.
&byname();

###
# Clean up and exit.
untie(%ALERTDB);




##
# Subroutine: Process records indexed by alert name
sub byname {
  my($name, $ids, @ids, $timestamp, $srchost, $srcport, $dsthost, $dstport);
  my($count);
  if (! open (SUMFILE, "> $LIBDIR/byname.csv")) {
      die "Could not create $LIBDIR/byname.csv:\n$!\n";
  }
  while (($name, $ids) = each(%ALERTNAMES)) {

    # Obtain alert IDs associated with the alert name
    @ids = split(/,/, $ids);

    # Save all entries relating to a particular alert into one file
    $count = $#ids + 1;
    if ($VERBOSE) {
	print $name , " ($count)...\n";
    }
    print SUMFILE "$name,$count\n";
    if (! open(OUTFILE, "> $LIBDIR/byname/$name.csv")) {
	warn "Could not write to $LIBDIR/$name:\n$!\n";
	next;
    }

    # Iterate through alert IDs for the current alert name,
    # retrieve details for each alert and write them to file.
    foreach (@ids) {
	$id = $_;
	($timestamp, $name, $srchost, $srcport, $dsthost, $dstport) =
	    split(/\*/, $ALERTDB{$id});
	print OUTFILE "$id,$timestamp,$srchost,$srcport,$dsthost,$dstport\n";
    }
    close(OUTFILE);
  }
  close(SUMFILE);
}


