#!/usr/bin/perl

###
# Parse a Berkley DB file of Snort alerts, produced by
# snorta-txt2bdb and create directories of specific alerts
# indexed for the particular destination host.

###
# Load necessary libraries.
require "init.pl";

# Host we want to retrieve data for.
$THEHOST = "MY.NET.253.12";

###
# Initialize input alert database.
if (! -r $ALERTDB) {
    die "Cannot read $ALERTDB. Exiting.\n$!\n";
}
if (! tie(%ALERTDB, "DB_File", $ALERTDB)) {
    die "Could not initialize input alert database $ALERTDB:\n$!\n";
}

###
# Process every entry in the input alert database
$counter = 0;
$| = 1;
if (! open(OUTALERTS, "> $LIBDIR/bydsthost/$THEHOST.csv")) {
    die "Could not open $LIBDIR/bydsthost/$THEHOST.csv:\n$!";
}
while (($id, $alert) = each(%ALERTDB)) {

    # Print progress indicator as records are processed
    if ($VERBOSE && (! ($counter % 500))) {
	print ".";
    }
    $counter++;

    # Parse database entries into distinct fields.
    ($timestamp, $name, $srchost, $srcport, $dsthost, $dstport) =
	split(/\*/, $alert);

    # Ignore port scans that sneaked into the alert file.
    next if ($name =~ /^spp_portscan:/);

    # Obtain destination  network (Class C) of the alert. If we match purely
    # by host we would miss slight variations in addresses such as
    # those in dial-up accounts.
    next if ($dsthost eq "");

    if ($dsthost eq $THEHOST) {
	print OUTALERTS "$id,$timestamp,$name,$srchost,$srcport,$dsthost,$dstport\n";
    }

}
print "\n" if ($VERBOSE);


###
# Clean up and exit.
close(OUTALERTS);
untie(%ALERTDB);

