#!/usr/bin/perl

###
# Parse a Berkley DB file of Snort scans, produced by
# snorts-txt2bdb and create a file with counts of occurrence of each network.

###
# Load necessary libraries.
require "init.pl";


###
# Initialize input alert database.
if (! -r $SCANDB) {
    die "Cannot read $SCANDB. Exiting.\n$!\n";
}
if (! tie(%SCANDB, "DB_File", $SCANDB)) {
    die "Could not initialize input scans database $SCANDB:\n$!\n";
}

###
# Process every entry in the input scans database
$counter = 0;
$| = 1;
%SCANNETS = ();
while (($id, $alert) = each(%SCANDB)) {

    # Print progress indicator as records are processed
    if ($VERBOSE && (! ($counter % 500))) {
	print ".";
    }
    $counter++;

    # Parse database entries into distinct fields.
    ($timestamp, $name, $srchost, $srcport, $dsthost, $dstport) =
	split(/\|/, $alert);

    # Obtain destination  network (Class C) of the alert. If we match purely
    # by host we would miss slight variations in addresses such as
    # those in dial-up accounts.
    next if ($dsthost eq "");

    # Keep track of scans per destination host.
    $SCANNETS{$dsthost} .= "$id,";

}
print "\n" if ($VERBOSE);

# Group scanss by destination host and write them to separate files
# for each host to allow them to be later imported as a spreadsheet.
# Create a file with counts of occurrence of each destination host.
&byhost();

###
# Clean up and exit.
untie(%SCANDB);




##
# Subroutine: Process records indexed by destination host.
sub byhost {
  my($name, $ids, @ids, $timestamp, $srchost, $srcport, $dsthost, $dstport);
  my($count);
  if (! open (SUMFILE, "> $LIBDIR/sbydsthost.csv")) {
      die "Could not create $LIBDIR/sbydsthost.csv:\n$!\n";
  }
  while (($host, $ids) = each(%SCANNETS)) {

    # Obtain alert IDs associated with the destination host
    @ids = split(/,/, $ids);

    $count = $#ids + 1;
    if ($VERBOSE) {
	print $host , " ($count)...\n";
    }
    print SUMFILE "$host,$count\n";
  }
  close(SUMFILE);
}


