#!/usr/bin/perl

###
# Parse a Berkley DB file of Snort alerts, produced by
# snorts-txt2bdb and create directories of scans
# indexed by the destination host of the scan.
# Also creates a separate file with counts of occurrence of each destination host.

###
# Load necessary libraries.
require "init.pl";


###
# Initialize input scan database.
if (! -r $SCANDB) {
    die "Cannot read $SCANDB. Exiting.\n$!\n";
}
if (! tie(%SCANDB, "DB_File", $SCANDB)) {
    die "Could not initialize input scan database $SCANDB:\n$!\n";
}

###
# Process every entry in the input scan database
$counter = 0;
$| = 1;
%SCANNETS = ();
while (($id, $scan) = each(%SCANDB)) {

    # Print progress indicator as records are processed
    if ($VERBOSE && (! ($counter % 500))) {
	print ".";
    }
    $counter++;

    # Parse database entries into distinct fields.
    ($timestamp, $name, $srchost, $srcport, $dsthost, $dstport) =
	split(/\|/, $scan);

    if (($dsthost eq "MY.NET.101.89") || ($dsthost eq "MY.NET.70.234") ||
	($dsthost eq "MY.NET.179.78") || ($dsthost eq "MY.NET.97.73") ||
	($dsthost eq "MY.NET.14.1")) {
      $SCANNETS{$dsthost} .= "$id,";
    }

}
print "\n" if ($VERBOSE);

# Group scans by destination host and write them to separate files
# for each host to allow them to be later imported as a spreadsheet.
# Also create a separate file with counts of occurrence of each host.
&byhost();

###
# Clean up and exit.
untie(%SCANDB);




##
# Subroutine: Process records indexed by destination host.
sub byhost {
  my($name, $ids, @ids, $timestamp, $srchost, $srcport, $dsthost, $dstport);
  my($count);
  while (($host, $ids) = each(%SCANNETS)) {

    # Obtain scan IDs associated with the host
    @ids = split(/,/, $ids);

    # Save all entries relating to a particular scan into one file
    $count = $#ids + 1;
    if ($VERBOSE) {
	print $host, " ($count)...\n";
    }
    if (! open(OUTFILE, "> $LIBDIR/sbydsthost/$host.csv")) {
	warn "Could not write to $LIBDIR/sbydsthost/$host:\n$!\n";
	next;
    }

    # Iterate through scan IDs for the current host,
    # retrieve details for each scan and write them to file.
    foreach (@ids) {
	$id = $_;
	($timestamp, $name, $srchost, $srcport, $dsthost, $dstport) =
	    split(/\|/, $SCANDB{$id});
	print OUTFILE "$id,$timestamp,$name,$srchost,$srcport,$dsthost,$dstport\n";
    }
    close(OUTFILE);
  }
}


