#!/usr/bin/perl

###
# Parse a Berkley DB file of Snort scans, produced by
# snorts-txt2bdb and create a file with counts of scans originating
# from MY.NET

###
# Load necessary libraries.
require "init.pl";


###
# Initialize input scans database.
if (! -r $SCANDB) {
    die "Cannot read $SCANDB. Exiting.\n$!\n";
}
if (! tie(%SCANDB, "DB_File", $SCANDB)) {
    die "Could not initialize input scans database $SCANDB:\n$!\n";
}

###
# Process every entry in the input scans database
$counter = 0;
$| = 1;
%SCANNETS = ();
while (($id, $alert) = each(%SCANDB)) {

    # Print progress indicator as records are processed
    if ($VERBOSE && (! ($counter % 500))) {
	print ".";
    }
    $counter++;

    # Parse database entries into distinct fields.
    ($timestamp, $name, $srchost, $srcport, $dsthost, $dstport) =
	split(/\|/, $alert);

    # Obtain source host of the scan.
    next if ($srchost eq "");

    # Keep track of scans per source host if the source is MY.NET
    if ($srchost =~ /^MY\.NET\./) {
	$SCANNETS{$srchost} .= "$id,";
    }

}
print "\n" if ($VERBOSE);

# Group scanss by source host and write them to separate files
# for each host to allow them to be later imported as a spreadsheet.
# Create a file with counts of occurrence of each source host.
&byhost();

###
# Clean up and exit.
untie(%SCANDB);




##
# Subroutine: Process records indexed by source host.
sub byhost {
  my($name, $ids, @ids, $timestamp, $srchost, $srcport, $dsthost, $dstport);
  my($count);
  if (! open (SUMFILE, "> $LIBDIR/ssrcmynet.csv")) {
      die "Could not create $LIBDIR/ssrcmynet.csv:\n$!\n";
  }
  while (($host, $ids) = each(%SCANNETS)) {

    # Obtain alert IDs associated with the source host
    @ids = split(/,/, $ids);

    $count = $#ids + 1;
    if ($VERBOSE) {
	print $host , " ($count)...\n";
    }
    print SUMFILE "$host,$count\n";
  }
  close(SUMFILE);
}


